@@ -1,4 +1,5 @@
-glmm_final<-function(y,X,W,k,q_start,Delta_start,s,steps=1000,family,method,overdispersion,phi,nue=1,print.iter.final=FALSE)
+glmm_final<-function(y,X,W,k,q_start,Delta_start,s,steps=1000,family,method,overdispersion,phi,
+                     nue=1,print.iter.final=FALSE,eps.final=1e-5,Q.min=1e-13,Q.max=20,Q.fac=5)
 {
 N<-length(y)
 lin<-ncol(as.matrix(X))
@@ -9,7 +10,6 @@ Sigma<-as.vector(family$variance(Mu))
 if(overdispersion)
 Sigma<-Sigma*phi
 D<-as.vector(family$mu.eta(Eta))
-W0_inv<-D*1/Sigma*D
 
 if(print.iter.final)
 print(paste("Final Re-estimation Iteration ", 1,sep=""))
@@ -27,32 +27,33 @@ for(jf in 1:n)
 P1[(lin+(jf-1)*s+1):(lin+jf*s),(lin+(jf-1)*s+1):(lin+jf*s)]<-chol2inv(chol(q_start))
 }
 
-if(overdispersion)
-{
-E<-diag(N)
-M0<-Z_alles%*%chol2inv(chol((t(Z_alles)%*%(Z_alles*W0_inv))+P1))%*%t(Z_alles*W0_inv)
-phi<-(sum((y-Mu)^2/Mu))/(N-sum(diag(M0)))
-}
-
 Delta<-matrix(0,steps,(lin+s*n))
-Delta[1,]<-Delta_start
 
 Q<-list()
 Q[[1]]<-q_start
 
-score_vec<-rep(0,(lin+s*n))
-D<-as.vector(family$mu.eta(Eta))
-Delta_r<-rep(0,lin+s*n)
-
 l=1
 opt<-steps
 
 score_vec<-t(Z_alles)%*%((y-Mu)*D*1/Sigma)-P1%*%Delta[1,]
 F_gross<-t(Z_alles)%*%(Z_alles*D*1/Sigma*D)+P1
 
-InvFisher<-chol2inv(chol(F_gross))
+InvFisher<-try(chol2inv(chol(F_gross)),silent=TRUE)
+if(class(InvFisher)=="try-error")
+InvFisher<-solve(F_gross)  
+
+half.index<-0
+solve.test<-FALSE
 Delta_r<-InvFisher%*%score_vec
-Delta[1,]<-Delta[1,]+nue*Delta_r
+
+######### big while loop for testing if the update leads to Fisher matrix which can be inverted
+while(!solve.test)
+{  
+
+solve.test2<-FALSE  
+while(!solve.test2)
+{  
+Delta[1,]<-Delta_start+nue*(0.5^half.index)*Delta_r
 
 Eta<-Z_alles%*%Delta[1,]
 
@@ -60,27 +61,45 @@ Mu<-as.vector(family$linkinv(Eta))
 Sigma<-as.vector(family$variance(Mu))
 D<-as.vector(family$mu.eta(Eta))
 
+
+if (method=="EM" || overdispersion)
+{  
+F_gross<-t(Z_alles)%*%(Z_alles*D*1/Sigma*D)+P1
+InvFisher<-try(chol2inv(chol(F_gross)),silent=TRUE)
+  if(class(InvFisher)=="try-error")
+  InvFisher<-try(solve(F_gross),silent=TRUE)  
+  if(class(InvFisher)=="try-error")
+  {
+    half.index<-half.index+1  
+  }else{
+    solve.test2<-TRUE 
+}}else{
+  solve.test2<-TRUE
+}}
+
 if (method=="EM")
 {
-F_gross<-t(Z_alles)%*%(Z_alles*D*1/Sigma*D)+P1
-InvFisher<-chol2inv(chol(F_gross))
 ############################# Q updaten ################
 Q1<-InvFisher[(lin+1):(lin+s),(lin+1):(lin+s)]+Delta[1,(lin+1):(lin+s)]%*%t(Delta[1,(lin+1):(lin+s)])
 for (i in 2:n)
 Q1<-Q1+InvFisher[(lin+(i-1)*s+1):(lin+i*s),(lin+(i-1)*s+1):(lin+i*s)]+Delta[1,(lin+(i-1)*s+1):(lin+i*s)]%*%t(Delta[1,(lin+(i-1)*s+1):(lin+i*s)])
 Q1<-1/n*Q1
 }else{
-Eta_tilde<-Eta+(y-Mu)*1/D
+Eta_tilde<-Eta+(y-Mu)/D
 
 Betadach<-Delta[1,1:lin]
 
 if(s==1)
 {
-optim.obj<-bobyqa(sqrt(q_start),likelihood_bobyqa,D=D,Sigma=Sigma,X=X,X_aktuell=X,Eta_tilde=Eta_tilde,n=n,Betadach=Betadach,W=W,lower = 1e-13, upper=20)
+low <- (1/Q.fac)*Q.min
+upp <- Q.fac*Q.max
+optim.obj<-nlminb(sqrt(q_start),likelihood_nlminb,D=D,Sigma=Sigma,X=X,X_aktuell=X,
+                  Eta_tilde=Eta_tilde,n=n,Betadach=Betadach,W=W,
+                  lower = low, upper=upp)  
 Q1<-as.matrix(optim.obj$par)^2
 }else{
 q_start_vec<-c(diag(q_start),q_start[lower.tri(q_start)])
-up1<-min(20,50*max(q_start_vec))#[(s+1):(s*(s+1)*0.5)]))
+up1<-Q.fac*Q.max
 upp<-rep(up1,length(q_start_vec))
 low<-c(rep(0,s),rep(-up1,0.5*(s^2-s)))
 kkk_vec<-c(rep(-1,s),rep(0.5,0.5*(s^2-s)))
@@ -105,46 +124,87 @@ Q[[2]]<-Q1
 
 if(overdispersion)
 {
-Uu<-(E-(Z_alles*sqrt(Sigma*D*1/Sigma*D))%*%InvFisher%*%t(Z_alles*sqrt(D*1/Sigma*D*1/Sigma)))%*%(E-M0)
-FinalHat<-E-Uu
+FinalHat<-(Z_alles*sqrt(Sigma*D*1/Sigma*D))%*%InvFisher%*%t(Z_alles*sqrt(D*1/Sigma*D*1/Sigma))#E-Uu
 phi<-(sum((y-Mu)^2/Mu))/(N-sum(diag(FinalHat)))
 Sigma<-Sigma*phi
 }
 
+if(s==1)
+{
+  P1<-c(rep(0,lin),rep((Q1^(-1)),n*s))
+  P1<-diag(P1)
+}else{
+  P1<-matrix(0,lin+n*s,lin+n*s)
+  for(jf in 1:n)
+    P1[(lin+(jf-1)*s+1):(lin+jf*s),(lin+(jf-1)*s+1):(lin+jf*s)]<-chol2inv(chol(Q1))
+}
+
+score_vec<-t(Z_alles)%*%((y-Mu)*D*1/Sigma)-P1%*%Delta[1,]
+F_gross<-t(Z_alles)%*%(Z_alles*D*1/Sigma*D)+P1
+
+
+InvFisher<-try(chol2inv(chol(F_gross)),silent=TRUE)
+
+  if(class(InvFisher)=="try-error")
+  InvFisher<-try(solve(F_gross),silent=TRUE)  
+
+  if(class(InvFisher)=="try-error")
+  {
+    half.index<-half.index+1  
+  }else{
+    solve.test<-TRUE 
+  }
+}
+
+y_dach<-as.vector(family$linkinv(Eta))
+Dev_neu<-sum(family$dev.resids(y,y_dach,wt=rep(1,N))^2)
 
 ###############################################################################################################################################
-################################################################### Boost ###################################################################
+################################################################### Main Iterations ###################################################################
+eps<-eps.final*sqrt(length(Delta_r))
+
 for (l in 2:steps)
 {
 
 if(print.iter.final)
 print(paste("Final Re-estimation Iteration ", l,sep=""))
 
-  if(s==1)
-  {
-  P1<-c(rep(0,lin),rep((Q1^(-1)),n*s))
-  P1<-diag(P1)
-  }else{
-  P1<-matrix(0,lin+n*s,lin+n*s)
-  for(jf in 1:n)
-  P1[(lin+(jf-1)*s+1):(lin+jf*s),(lin+(jf-1)*s+1):(lin+jf*s)]<-chol2inv(chol(Q1))
-  }
+half.index<-0
+solve.test<-FALSE
 
-  score_vec<-t(Z_alles)%*%((y-Mu)*D*1/Sigma)-P1%*%Delta[l-1,]
-  F_gross<-t(Z_alles)%*%(Z_alles*D*1/Sigma*D)+P1
-
-InvFisher<-chol2inv(chol(F_gross))
 Delta_r<-InvFisher%*%score_vec
-Delta[l,]<-Delta[l-1,]+nue*Delta_r
+######### big while loop for testing if the update leads to Fisher matrix which can be inverted
+while(!solve.test)
+{  
+
+solve.test2<-FALSE  
+while(!solve.test2)
+{  
+Delta[l,]<-Delta[l-1,]+nue*(0.5^half.index)*Delta_r
+
 Eta<-Z_alles%*%Delta[l,]
 Mu<-as.vector(family$linkinv(Eta))
 Sigma<-as.vector(family$variance(Mu))
 D<-as.vector(family$mu.eta(Eta))
 
+if (method=="EM" || overdispersion)
+{  
+  F_gross<-t(Z_alles)%*%(Z_alles*D*1/Sigma*D)+P1
+  InvFisher<-try(chol2inv(chol(F_gross)),silent=TRUE)
+  if(class(InvFisher)=="try-error")
+    InvFisher<-try(solve(F_gross),silent=TRUE)  
+  if(class(InvFisher)=="try-error")
+  {
+    half.index<-half.index+1  
+  }else{
+    solve.test2<-TRUE 
+  }}else{
+    solve.test2<-TRUE
+}}
+
+
 if (method=="EM")
 {
-F_gross<-t(Z_alles)%*%(Z_alles*D*1/Sigma*D)+P1
-InvFisher<-chol2inv(chol(F_gross))
 ############################# Q update ################
 Q1<-InvFisher[(lin+1):(lin+s),(lin+1):(lin+s)]+Delta[l,(lin+1):(lin++s)]%*%t(Delta[l,(lin+1):(lin+s)])
 for (i in 2:n)
@@ -152,15 +212,22 @@ Q1<-Q1+InvFisher[(lin+(i-1)*s+1):(lin+i*s),(lin+(i-1)*s+1):(lin+i*s)]+Delta[l,(l
 
 Q1<-1/n*Q1
 }else{
-Eta_tilde<-Eta+(y-Mu)*1/D
+Eta_tilde<-Eta+(y-Mu)/D
 
 Betadach<-Delta[l,1:lin]
 
 if(s==1)
 {
-optim.obj<-bobyqa(sqrt(Q1),likelihood_bobyqa,D=D,Sigma=Sigma,X=X,X_aktuell=X,Eta_tilde=Eta_tilde,n=n,Betadach=Betadach,W=W, lower = 1e-13, upper = 20)
+upp<-max(upp,Q.fac*sqrt(Q1))
+low<-min(low,(1/Q.fac)*sqrt(Q1))
+optim.obj<-nlminb(sqrt(Q1),likelihood_nlminb,D=D,Sigma=Sigma,X=X,X_aktuell=X,
+                  Eta_tilde=Eta_tilde,n=n,Betadach=Betadach,W=W, 
+                  lower = low, upper = upp)
 Q1<-as.matrix(optim.obj$par)^2
 }else{
+up1<-max(up1,Q.fac*max(Q1))  
+upp<-rep(up1,length(q_start_vec))
+low<-c(rep(0,s),rep(-up1,0.5*(s^2-s)))
 Q1_vec<-c(diag(Q1),Q1[lower.tri(Q1)])
 optim.obj<-bobyqa(Q1_vec,likelihood,D=D,Sigma=Sigma,X=X,X_aktuell=X,Eta_tilde=Eta_tilde,n=n,s=s,k=k,Betadach=Betadach,W=W,lower=low,upper=upp)
 
@@ -182,45 +249,58 @@ for (ttt in 0:100)
 
 Q[[l+1]]<-Q1
 
+
 if(overdispersion)
 {
-Uu<-(E-(Z_alles*sqrt(Sigma*D*1/Sigma*D))%*%InvFisher%*%t(Z_alles*sqrt(D*1/Sigma*D*1/Sigma)))%*%Uu
-FinalHat<-E-Uu
+FinalHat<-(Z_alles*sqrt(Sigma*D*1/Sigma*D))%*%InvFisher%*%t(Z_alles*sqrt(D*1/Sigma*D*1/Sigma))#E-Uu
 phi<-(sum((y-Mu)^2/Mu))/(N-sum(diag(FinalHat)))
+Sigma<-Sigma*phi
 }
 
+if(s==1)
+{
+  P1<-c(rep(0,lin),rep((Q1^(-1)),n*s))
+  P1<-diag(P1)
+}else{
+  P1<-matrix(0,lin+n*s,lin+n*s)
+  for(jf in 1:n)
+    P1[(lin+(jf-1)*s+1):(lin+jf*s),(lin+(jf-1)*s+1):(lin+jf*s)]<-chol2inv(chol(Q1))
+}
+
+score_vec<-t(Z_alles)%*%((y-Mu)*D*1/Sigma)-P1%*%Delta[l,]
+F_gross<-t(Z_alles)%*%(Z_alles*D*1/Sigma*D)+P1
+
+InvFisher<-try(chol2inv(chol(F_gross)),silent=TRUE)
+  if(class(InvFisher)=="try-error")
+  InvFisher<-try(solve(F_gross),silent=TRUE)  
+  if(class(InvFisher)=="try-error")
+  {
+    half.index<-half.index+1  
+  }else{
+    solve.test<-TRUE 
+  }
+}
 
 kritval<-sqrt(sum((Delta[l-1,]-Delta[l,])^2))/sqrt(sum(Delta[l-1,]^2))
-if(kritval<1e-6)
-break
-
+if(kritval<eps)
+break           
+           
 if(l>2)
 {
 kritval2<-sqrt(sum((Delta[l-2,]-Delta[l,])^2))/sqrt(sum(Delta[l-2,]^2))
-if(kritval2<1e-6)
+if(kritval2<eps)
 break
 }}
 
-#print(paste("Final Iteration =", l,sep=""))
 
 
 opt<-l
 Deltafinal<-Delta[l,]
 Q_final<-Q[[l+1]]
 
-  if(s==1)
-  {
-  P1<-c(rep(0,lin),rep((Q_final^(-1)),n*s))
-  P1<-diag(P1)
-  }else{
-  P1<-matrix(0,lin+n*s,lin+n*s)
-  for(jf in 1:n)
-  P1[(lin+(jf-1)*s+1):(lin+jf*s),(lin+(jf-1)*s+1):(lin+jf*s)]<-chol2inv(chol(Q_final))
-  }
-F_gross<-t(Z_alles)%*%(Z_alles*D*1/Sigma*D)+P1
-Inv_F_opt<-chol2inv(chol(F_gross))
+Standard_errors<-sqrt(diag(InvFisher))
 
-Standard_errors<-sqrt(diag(Inv_F_opt))
+#FinalHat<-(Z_alles*sqrt(D*1/Sigma*D))%*%Inv_F_opt%*%t(Z_alles*sqrt(D*1/Sigma*D))
 
 ret.obj=list()
 ret.obj$opt<-opt
@@ -228,5 +308,6 @@ ret.obj$Delta<-Deltafinal
 ret.obj$Q<-Q_final
 ret.obj$Standard_errors<-Standard_errors
 ret.obj$phi<-phi
+#ret.obj$complexity<-sum(diag(FinalHat))
 return(ret.obj)
 }
